"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const createSegments = (path) => {
    const root = { name: "" };
    let acc = root;
    for (const name of path.split("/")) {
        const child = { name };
        acc.child = child;
        acc = child;
    }
    return root;
};
const serializeSegments = (segment) => {
    let result = segment.name;
    while (segment.child !== undefined) {
        segment = segment.child;
        result += `/${segment.name}`;
    }
    return result;
};
const emptyRouter = {
    name: "",
    children: {},
};
const addHandler = (router, { name, child }, handler) => {
    if (child === undefined) {
        return {
            name,
            handler,
            children: router.children,
        };
    }
    else {
        const childName = child.name.startsWith(":") ? ":" : child.name;
        const childRouter = router.children[childName] || emptyRouter;
        const newChildRouter = addHandler(childRouter, child, handler);
        return {
            name,
            handler: router.handler,
            children: Object.assign(Object.assign({}, router.children), { [childName]: newChildRouter }),
        };
    }
};
const findHandler = (router, segment, params) => {
    var _a;
    const { child } = segment;
    const { handler } = router;
    if (child === undefined && handler !== undefined) {
        return [handler, params];
    }
    if (child === undefined) {
        return undefined;
    }
    const nameRouter = router.children[child.name];
    const nameRouterResult = nameRouter !== undefined
        ? findHandler(nameRouter, child, params)
        : undefined;
    if (nameRouterResult !== undefined) {
        return nameRouterResult;
    }
    const paramRouter = router.children[":"];
    const paramRouterResult = paramRouter !== undefined
        ? findHandler(paramRouter, child, Object.assign(Object.assign({}, params), { [paramRouter.name]: child.name }))
        : undefined;
    if (paramRouterResult !== undefined) {
        return paramRouterResult;
    }
    const globRouter = router.children["*"];
    const globRouterHandler = (_a = globRouter) === null || _a === void 0 ? void 0 : _a.handler;
    if (globRouterHandler !== undefined) {
        const newParams = Object.assign(Object.assign({}, params), { ["*"]: serializeSegments(child) });
        return [globRouterHandler, newParams];
    }
    return undefined;
};
exports.createRouter = (routes, notFoundHandler) => {
    const router = Object.entries(routes).reduce((acc, [path, handler]) => addHandler(acc, createSegments(path), handler), emptyRouter);
    return (request) => {
        const segments = createSegments(request.uri.pathname);
        const result = findHandler(router, segments, {});
        if (result !== undefined) {
            const [handler, params] = result;
            const requestWithParams = Object.assign(Object.assign({}, request), { params });
            return handler(requestWithParams);
        }
        else {
            return notFoundHandler(request);
        }
    };
};
//# sourceMappingURL=index.js.map